#! /bin/sh


error_help_exit() {
  echo -e "\nUpdater ver 3.0\nUsage sample:\n\
sh updater.sh ./soft.tgz \n\
" ;
  exit $2;
}


! [ "$#" -eq "1" ] && { error_help_exit 'Error! No needed parameters!' 1 ; }

TMP=`mktemp -d /tmp/temp.XXXXXX || { error_help_exit 'Error! Cant create tmp dir' 2; }`

error_clean_retry() {
	{
	echo;
	echo -e $1;
	echo;
	echo -n 'Clearing temporary files and downloads...';
	rmmod mtdrw;
	[ -d "$TMP" ] && rm -rf $TMP;
	echo Ok;
	echo 'You may try again later...';
	} >&2
	exit $2;
}

error_stop() {
	{
	echo;
	echo -e $1;
	echo;
	echo -e 'I don`t know what to do now.\nYour FLASH is in broken state.\nYou can`t reboot. You may try to recover some stages manually.';
	} >&2
	exit $2;
}

error_clean_reboot() {
	return 0;
}

ddd() {
	local DDD_FILE=`mktemp $TMP/dd_buf.XXXXXX || { error_stop 'Error! Cant create tmp file' 3; }`
	local DDD_C=0;
	local DDD_S=$1;
	local DDD_CC=-1;
	local DDD_EC=0;
	shift 1;
	while [ "$DDD_EC" -eq "0" -a ! "$DDD_C" -eq "$DDD_S"  -a ! "$DDD_CC" -eq "0" ] ; do
		dd $@  count=1 bs=$(($DDD_S-$DDD_C)) of=$DDD_FILE 2>/dev/null;
		DDD_CC=`stat -c %s $DDD_FILE`;
		cat $DDD_FILE;
		DDD_C=$(($DDD_C+$DDD_CC));
		DDD_EC=$?;
	done;
	[ -f "$DDD_FILE" ] && rm -rf "$DDD_FILE";
	return $DDD_EC
}

NSGSOFT=`realpath $1`
echo
echo
echo !!! This program will update your NSG700 u-boot and kernel, lives configurations as is !!!
echo
echo
#read E || ( echo 'Aborted!' && exit 1 ) 
#[ "$E" != "" ] && ( echo 'Aborted!' && exit 1 ) 

KVER=$(uname -r)

if ! [ "2.4.27-vrs1_at91-nsg" == $KVER ];  then
    echo Error! This program is for NSG700 Linux 1.0 only !!!
    exit 6
fi

BUF_FILE=`mktemp $TMP/buf_file.XXXXXX || { error_clean_retry 'Error! Cant create tmp file' 4; }`
cd $TMP || { error_clean_retry 'Error! Cant change dir to tmpdir:'"$TMP" 5; }

NSG_DEVICE=`head -n1 /proc/sys/nsg/chassis || { error_clean_retry 'Error! Cant read current chassis.' 6; }`
NSG_VERSION=`head -n1 /proc/sys/nsg/.sys/release || { error_clean_retry 'Error! Cant read current release.'  7; }`
ETH_ADDR=`head -c 134096 /dev/mtd/0 | tail -c 3024 | grep '^ethaddr=' | grep -oE '[^=]*$' | head -n 1 || { error_clean_retry 'Error! Cant read MAC addr.' 8; }`
SERIAL_NUM=`head -c 134096 /dev/mtd/0 | tail -c 3024 | grep '^serial#=' | grep -oE '[^=]*$' | head -n 1  || { error_clean_retry 'Error! Cant read SERIAL NUM.' 9; }`

echo List of files in software pack:
[ -f "$NSGSOFT" ] && tar -tvzf  "$NSGSOFT" && ln -s "$NSGSOFT" ./soft.tgz ||  { error_clean_retry 'Error! Cant read/find software pack file' 10; }

NEW_NSG_DEVICE=`echo $NSG_DEVICE | \
	sed 's/^nsg700-4a$/NSG700_4AU_v5/' | \
	sed 's/^nsg700-8a$/NSG700_8AU/' | \
	sed 's/^nsg700-12a$/NSG700_12AU/' | \
	sed 's/^nsg700v6$/NSG700_4AU_v6/' | \
	sed 's/^nsg700v6m$/NSG700_4AU_v6m/' | \
	sed 's/^nsg700v7$/NSG700_4AU_v7/' | \
	sed 's/^nsg700-4a-fiber$/NSG700_4AU_v5/' | \
	sed 's/^nsg700v6-fiber$/NSG700_4AU_v6f/' | \
	sed 's/^nsg700v6m-fiber$/NSG700_4AU_v6f/' | \
	sed 's/^nsg700v7-fiber$/NSG700_4AU_v7/' `


echo $NEW_NSG_DEVICE | grep '^NSG700_' >/dev/null ||  { error_clean_retry "Error! Unsupported device:$NSG_DEVICE." 14; }

[ -n "$FORCE_SERIAL_NUM" ] && SERIAL_NUM=$FORCE_SERIAL_NUM
[ -n "$FORCE_ETH_ADDR" ] && ETH_ADDR=$FORCE_ETH_ADDR

NEW_SERIAL_NUM=`echo "000000""$SERIAL_NUM" | head -c 10`
NEW_ETH_ADDR="$ETH_ADDR"

echo

TGZ=`tar -tzvf ./soft.tgz`
mtdrw=`echo "$TGZ" | grep -oE '\s([.][/])?mtdrw[.]o$'`
nsg700core=`echo "$TGZ" | grep -oE '\s([.][/])?nsg700-linux-core[.]bin$'`
nsg700uboot=`echo "$TGZ" | grep -oE '\s([.][/])?u-boot-1.1.4-nsg700[.]bin$'`

[ -n "${mtdrw}" ] || { error_clean_retry 'Error! No mtdrw.o in or bad archive '"${NSGSOFT}" 17; }
[ -n "${nsg700core}" ] || { error_clean_retry 'Error! No nsg700-linux-core.bin in or bad archive '"${NSGSOFT}" 18; }
[ -n "${nsg700uboot}" ] || { error_clean_retry 'Error! No u-boot-1.1.4.-nsg700.bin in or bad archive '"${NSGSOFT}" 19; }

echo
echo 'Extracting mtd driver mtdrw.o:'
tar -xzf  soft.tgz ${mtdrw} && echo OK || { error_clean_retry 'Error! Cant extract mtdrw.o' 20; }
echo

echo Current Board info:
echo nsg_device="$NSG_DEVICE"
echo serial_num="$SERIAL_NUM"
echo ethaddr="$ETH_ADDR"
echo software version="$NSG_VERSION"
echo

echo Press AnyKey to LAST SAFE abort or Enter to continue.
[ -n "$FORCE_Y" ] || { read E || { error_clean_retry 'Aborted!' 22; }
[ "$E" != "" ] && { error_clean_retry 'Aborted!' 23; }
}

echo
insmod ./mtdrw.o && echo MTD driver loaded OK || { error_clean_retry "Error! Can't insmod mtd driver: mtdrw.o" 24; }
echo
echo !!! DON"'"T DO ANYTHING !!!
echo !!! WAIT FOT JOB FINISHED !!!
echo !!! AND RESTARTED !!!
echo

echo Updating core:
echo Clearing...
{ flash_eraseall /dev/mtd/4 || { error_clean_retry "Error! Cant erase /dev/mtd/4.\nError! Cant erase core flash. Something wrong, try again or flash router from uboot..."  25; } ; } | tr -d "\n"
echo -e '\r100% done. OK                                         '
SIZE=`echo "$TGZ" | grep '\bnsg700-linux-core.bin\b' | grep -oE '[0-9]{5,}' || { error_stop "Error! Cant find size of nsg700-linux-core.bin" 28; }`
echo Writing nsg700-linux-core.bin...
tar -xzOf soft.tgz ${nsg700core} | { C=0 ; RES="-1"; while [ "$RES" -ne "0" ]; do  ddd 131072 >$BUF_FILE; RES=`stat -c %s $BUF_FILE` ; cat $BUF_FILE ; C=$(($C+$RES)); echo -e -n '\r'$C'-'$((100*$C/$SIZE))'%' >&2 ;  done; } >/dev/mtd/4 || { error_clean_retry "Error! Cant write to /dev/mtd/4.\nVery bad :(\nTry again or flash router from uboot..." 29; }
echo
echo Writen OK
echo

echo Updating uboot:
echo Clearing...
{ flash_eraseall /dev/mtd/1 || { error_stop "Error! Cant erase /dev/mtd/1.\nVery bad :(\nI think device is dead..." 27; }; } | tr -d "\n"
echo -e '\r100% done. OK                                          '
SIZE=`echo "$TGZ" | grep '\bu-boot-1.1.4-nsg700.bin\b' | grep -oE '[0-9]{5,}' || { error_stop "Error! Cant find size of u-boot-1.1.4-nsg700.bin" 28; }`
echo Writing u-boot-1.1.4-nsg700.bin...
tar -xzOf soft.tgz ${nsg700uboot} | { C=0 ; RES="-1"; while [ "$RES" -ne "0" ]; do  ddd 131072 >$BUF_FILE; RES=`stat -c %s $BUF_FILE` ; cat $BUF_FILE ; C=$(($C+$RES)); echo -e -n '\r'$C'-'$((100*$C/$SIZE))'%' >&2 ;  done; } >/dev/mtd/1 || { error_stop "Error! Cant write to /dev/mtd/1.\nVery bad :(\nI think device is dead, try again or flash router in emergency restoring mode..." 29; }
echo
echo Writen OK
echo

sync
sync
sync

echo "All done!"
echo "Rebooting now."
echo
sync
echo -n 10; sleep 1;
echo -n -e '\r9 '; sleep 1;
echo -n -e '\r8'; sleep 1;
echo -n -e '\r7'; sleep 1;
echo -n -e '\r6'; sleep 1;
echo -n -e '\r5'; sleep 1;
sync
echo -n -e '\r4'; sleep 1;
echo -n -e '\r3'; sleep 1;
echo -n -e '\r2'; sleep 1;
echo -n -e '\r1'; sleep 1;
sync

reboot

